/**
 * Scan and load controllers
 */

require("../utils/annotation");

var readdirp = require("readdirp");
var configs = require("../conf/configs");
var logger = require("../utils/logger").outputLogger;
var controllerProcessor = require("./processor");

/**
 * Scan a directory and load all controllers found
 *
 * @param {string} root - Scan directory
 * @param {function} callback - Callback function with signature fn(err: Error, result: Object)
 */
function scan(root, callback)
{
	var result = {};

    logger.info("Scanning for controllers...");

	readdirp({ root: root, fileFilter: "*.controller.js" })
		.on("data", function (entry) {
			var Controller = require(entry.fullPath);
			if (!Controller.hasAnnotation("controller")) {
				return;
			}

			var info = Controller.getAnnotationData("controller");
			var c = new Controller();
            var controllerData = {
				controller: c,
				path: entry.fullPath
			};

			if (result[info.urlPattern]) {
				return;
			}

			result[info.urlPattern] = controllerData;

			logger.info("Loaded controller at " + entry.path);

			if (typeof(c.init) === "function") {
				c.init({
					configs: configs
				});
			}

            if (configs.precompileViews === true) {
				var doProcessView = function (viewPath) {
					controllerProcessor.processView(viewInfo, controllerData.path, {}, {}, true, function (err, data) {
						if (err) {
							console.dir(err);
							logger.error("Error occured when compiling view page " + viewPath);
							return;
						}

						logger.info("Pre-compiled view page " + viewPath + " for controller " + controllerData.path);
					});
				};

                for (var key in c) {
					if (!c.hasOwnProperty(key)) {
						continue;
					}

                    if (typeof(c[key]) !== "function") {
                        continue;
                    }

                    var func = c[key];
                    if (!func.hasAnnotation("view")) {
                        continue;
                    }

                    var viewInfo = func.getAnnotationData("view");
					doProcessView(viewInfo.path);
                }
            }
		})
		.on("end", function () {
            logger.info("All controllers loaded.");
			callback(null, result);
		})
		.on("error", function (err) {
			callback(err);
		});
}

exports.scan = scan;
