/**
 * HTTP Server for node-bubble
 */

var http = require("http");

var configs = require("../conf/configs");

var controllerLoader = require("../controller/loader");
var filterLoader = require("../filter/loader");
var router = require("./router");
var errors = require("../errors");
var logger = require("../utils/logger").outputLogger;
var readdirp = require("readdirp");
var path = require("path");

var controllerList = null;
var filterList = null;
var authenticatorList = {};

function processRequest(request, response)
{
	var needResp = true;

	response.on("finish", function () {
		needResp = false;
	});

	router.route(request, response, controllerList, filterList,
		function (err, stream, mimeType, length, skipHeader) {
			if (err) {
				if (err instanceof errors.FileNotFound) {
					logger.warn("FileNotFoundError: " + err.message);

					if (needResp) {
						response.writeHead(404);
						response.write(err.message);
						response.end();
					}

					return;
				}

				logger.error("Error: " + err.name + ": " + err.message);
				logger.error("Stacktrace:");
				logger.error(err.stack);

				if (needResp) {
					response.writeHead(500);
					response.write(err.message);
					response.end();
				}

				return;
			}

			if (needResp) {
				if (!skipHeader) {
					response.writeHead(200, {
						"Content-Type": mimeType,
						"Content-Length": length
					});
				}

				if (stream) {
					stream.pipe(response);
				} else {
					response.end();
				}
			}
		});
}

/**
 * Start HTTP server
 *
 * @param {function} callback - Callback function with signature fn(configs: Object)
 */
function start(callback)
{
	controllerLoader.scan(configs.webRoot, function (err, result) {
		if (err) {
			logger.error("Error: " + err);
			logger.error(err.stack);
			callback(err);
			return;
		}

		controllerList = result;
	});

	filterLoader.scan(configs.webRoot, function (err, result) {
		if (err) {
			logger.error("Error: " + err);
			logger.error(err.stack);
			callback(err);
			return;
		}

		filterList = result;
	});

	readdirp({ root: configs.webRoot, fileFilter: "*.auth.js" })
		.on("data", function (entry) {
			if (!authenticatorList[entry.fullPath]) {
				var authenticator = require(entry.fullPath);
				authenticatorList[entry.fullPath] = true;
				logger.info("Loaded authenticator at " + entry.path);
			}
		})
		.on("end", function () {
			logger.info("All authenticators loaded.");
		})
		.on("error", function (err) {
			logger.error("Error: " + err);
			logger.error(err.stack);
		});

	var server = http.createServer(processRequest);
	server.listen(configs.port);

	logger.info("Server " + configs.host + " in web root " + path.resolve(configs.webRoot) + " started at port " +
		configs.port);

	if (callback) {
		callback(configs);
	}
}

exports.start = start;
