/**
 * Router for HTTP server
 */

var urlParser = require("url");
var fs = require("fs");
var path = require("path");
var mime = require("mime");

var configs = require("../conf/configs");

var controllerProcessor = require("../controller/processor");
var errors = require("../errors");
var logger = require("../utils/logger").outputLogger;
var accessLogger = require("../utils/logger").accessLogger;
var requestUtil = require("../utils/request");

/**
 * Route URL to file
 *
 * @param {object} urlData - The URL data to route
 * @param {object} request - The request object
 * @param {object} response - The response object
 * @param {function} callback - Callback function with signature fn(err: Error, stream: ReadStream,
 *								mimeType: string, length: integer)
 */
function routeToFile(urlData, request, response, callback)
{
	var url = urlData.pathname;
	var filePath;

	var mapFound = false;
	var allow = false;
	var realPath;

	for (var key in configs.fileUrlMap) {
		if (url.indexOf(key) === 0) {
			url = url.replace(key, configs.fileUrlMap[key]);
			realPath = path.resolve(path.join("/", configs.fileUrlMap[key]));
			mapFound = true;
			break;
		}
	}

	if (mapFound) {
		filePath = path.resolve(path.join("/", url));
		allow = (filePath.indexOf(realPath) === 0);
	} else {
		filePath = path.resolve(path.join(configs.webRoot, url));

		if (filePath.indexOf(path.resolve(configs.webRoot)) < 0) {
			logger.warn("Path " + filePath + " is out of web root " + configs.webRoot + ", denied!");
			callback(new errors.FileNotFound("Requested url " + urlData.pathname + " not found!"));
			return;
		}

		for (var i = 0; i < configs.accessible.length; i++) {
			var allowedPath = path.resolve(path.join(configs.webRoot, configs.accessible[i]));
			if (filePath.indexOf(allowedPath) === 0) {
				allow = true;
			}
		}
	}

	if (!allow) {
		logger.warn("Path " + filePath + " is not allowed to access!");
		return callback(new errors.FileNotFound("Requested url " + urlData.pathname + " not found!"));
	}

	fs.stat(filePath, function (err, stats) {
		if (err) {
			return callback(err);
		}

		if (request.headers["if-modified-since"]) {
			var modifyCheckDate = Date.parse(request.headers["if-modified-since"]);

			if (modifyCheckDate >= stats.mtime) {
				response.writeHead(304);
				response.end();
				callback(null, null, null, 0, true);
				return;
			}
		}

		response.setHeader("Last-Modified", stats.mtime.toUTCString());

		/*console.info("Requested url " + urlData.pathname + " mapped to file " + filePath
				+ ", length " + stats.size);*/

		callback(null, fs.createReadStream(filePath), mime.lookup(filePath), stats.size);
	});
}

/**
 * Route URL to controller
 *
 * @param {object} request - The request
 * @param {object} response - The response
 * @param {string} urlData - The URL data to route
 * @param {object} controllerList - List of controllers
 * @param {function} callback - Callback function with signature fn(err: Error, stream: ReadStream,
 *								mimeType: string, length: integer)
 */
function routeToController(request, response, urlData, controllerList, callback)
{
	var c = null;

	for (var urlPattern in controllerList) {
		if (new RegExp(urlPattern, "i").test(urlData.pathname)) {
			c = controllerList[urlPattern];

			if (controllerProcessor.canProcess(urlData, c.controller)) {
				//console.info("Request " + urlData.pathname + " routed to controller " + controller);
				break;
			} else {
				c = null;
			}
		}
	}

	if (c == null) {
		callback(new errors.NoRouteToController("Request " + urlData.pathname + " cannot be routed to any controller!"));

		return;
	}

	controllerProcessor.process(request, response, urlData, c, callback);
}

/**
 * Route request to processors
 *
 * @param {object} request - The request to route
 * @param {object} respones - The response
 * @param {object} controllerList - List of controllers
 * @param {Array} filterList - List of filters
 * @param {function} callback - Callback function with signature fn(err: Error, stream: ReadStream,
 *								mimeType: string, length: integer)
 */
function route(request, response, controllerList, filterList, callback)
{
	//console.info("Requested url: " + request.url);
	accessLogger.info(requestUtil.getRequestIP(request) + " " + request.method + " " + request.url);

	if ((filterList) && (filterList.length > 0)) {
		for (var i = 0; i < filterList.length; i++) {
			if (new RegExp(filterList[i].info.urlPattern, "i").test(request.url)) {
				if (filterList[i].filter.filter(request, response) === false) {
					logger.info("Request cut by filter " + filterList[i]);
					return;
				}
			}
		}
	}

	//console.info("After filters: " + request.url);

	var urlData = urlParser.parse(request.url, true);
	if (urlData.pathname == "/") {
		urlData.pathname = path.join(urlData.pathname, configs.defaultUrl);
	}

	routeToController(request, response, urlData, controllerList,
		function (err, stream, mimeType, length) {
			if (err) {
				if (err instanceof errors.NoRouteToController) {
					routeToFile(urlData, request, response, callback);
					return;
				}

				callback(err);
				return;
			}

			callback(null, stream, mimeType, length);
		});
}

exports.route = route;
