/**
 * Request processing utils
 */

var configs = require("../conf/configs");
var errors = require("../errors");
var querystring = require("querystring");
var urlParser = require("url");
var multiparty = require("multiparty");

/**
 * Get data from request
 *
 * @param {object} request - The request object
 * @param {function} callback - Callback with signature fn(err: Error, data: Object, files: Object)
 */
function getRequestData(request, callback)
{
	if (request.method === "GET") {
		callback(null, urlParser.parse(request.url, true).query);
	} else if (request.method === "POST") {
		var buf = "";
		var isMultipart = false;

		if ((request.headers["content-type"]) && (request.headers["content-type"].indexOf("multipart/form-data") >= 0)) {
			isMultipart = true;
		}

		if (!isMultipart) {
			request.setEncoding("utf8");
			request.on("data", function (data) {
				buf += data;

				if (buf.length > configs.maxPostSize) {
					request.connection.destroy();
					callback(new errors.RequestTooLong("Request exceeded max POST length limit!"));
					return;
				}
			})
			.on("end", function () {
				callback(null, querystring.parse(buf));
			});
		} else {
			var form = new multiparty.Form({ maxFilesSize: configs.maxPostSize });
			form.parse(request, function (err, fields, files) {
				if (err) {
					callback(err);
					return;
				}

				var params = urlParser.parse(request.url, true).query;
				for (var key in params) {
					if (!params.hasOwnProperty(key)) {
						continue;
					}

					if (fields[key]) {
						if (!(fields[key] instanceof Array)) {
							var a = [ fields[key] ];
							fields[key] = a;
						}

						fields[key].push(params[key]);
					} else {
						fields[key] = params[key];
					}
				}

				callback(null, fields, files);
			});
		}
	}
}

function getRequestIP(req)
{
	try {
		return req.headers['x-forwarded-for'] || req.connection.remoteAddress || req.socket.remoteAddress ||
			req.connection.socket.remoteAddress;
	} catch (err) {
		return "";
	}
}

exports.getRequestData = getRequestData;
exports.getRequestIP = getRequestIP;
